<?php
/**
 * This file is part of workerman.
 *
 * Licensed under The MIT License
 * For full copyright and license information, please see the MIT-LICENSE.txt
 * Redistributions of files must retain the above copyright notice.
 *
 * @author walkor<walkor@workerman.net>
 * @copyright walkor<walkor@workerman.net>
 * @link http://www.workerman.net/
 * @license http://www.opensource.org/licenses/mit-license.php MIT License
 */

/**
 * 用于检测业务代码死循环或者长时间阻塞等问题
 * 如果发现业务卡死，可以将下面declare打开（去掉//注释），并执行php start.php reload
 * 然后观察一段时间workerman.log看是否有process_timeout异常
 */

//declare(ticks=1);

use \GatewayWorker\Lib\Gateway;
use App\Common\LogTrait;
use App\Common\HelperTrait;
use App\Common\DecodeTrait;

/**
 * 主逻辑
 * 主要是处理 onConnect onMessage onClose 三个方法
 * onConnect 和 onClose 如果不需要可以不用实现并删除
 */
class Events
{
    use LogTrait, HelperTrait, DecodeTrait, \App\Common\DBTrait, \App\Models\DeviceTrailModel;

    public static function onWorkerStart($businessWorker)
    {
        self::initDataBase([1, 2]);
    }

    /**
     * 当客户端连接时触发
     * 如果业务不需此回调可以删除onConnect
     *
     * @param int $client_id 连接id
     */
    public static function onConnect($client_id)
    {
    }

    /**
     * 当客户端发来消息时触发
     * @param int $client_id 连接id
     * @param mixed $message 具体消息
     */
    public static function onMessage($client_id, $message)
    {
        $message = bin2hex($message);
        self::log($message, 'onMessage');
        //处理粘包问题
        $mes = [];
        $data = explode('0d0a', strtolower($message));
        $ys = '';
        foreach ($data as $d) {
            $da = str_split(str_replace(' ', '', trim($ys . $d)), 2);
            if (count($da) <= 2) break;
            $length = $da[2];

            $content = array_slice($da, 3, -1);

            if (count($content) === (int)$length) {
                $mes[] = $d . '0d0a';
            } else {
                $ys .= $d;
            }
        }

        foreach ($mes as $m) {
            self::handleMessage($client_id, $m);
        }

//        $data = self::str2arr(strtolower($message));
//        $type = $data[1];
//
//        //校检位数
//        $length = $data[2];
//        $content = array_slice($data, 3, -3);
//        if (count($content) !== (int)$length) {
//            self::log("msg length is error!");
//            return false;
//        }
//
//        //校检是否正确
////        $header = array_slice($data, 0, 3);
//        $checkCode = array_slice($data, -3, 1)[0];
//
//        $checkCodeRe = self::sumDexArr(array_slice($data, 0, -3));
//
//        if ($checkCodeRe !== $checkCode) {
//            self::log("check is error!");
//            return false;
//        }
//
//        $box_no = self::decodeBoxNo(array_slice($content, 0, 15));
//
//        if (!isset($_SESSION['box_no'])) {
//            $_SESSION['box_no'] = $box_no;
//            self::onLine($_SESSION['box_no']);
//        }
//
//        if (!Gateway::getUidByClientId($client_id)) {
//            Gateway::bindUid($client_id, $box_no);
//        }
//
//
////        self::response('a5', 'a6');
//        $param = [
//            'db' => self::$db,
//            'redis' => self::$redis
//        ];
//        $body = $content;
//        switch ($type) {
//            case 'a5':
//                //数据上传
//                $responseBody = (new \App\Servers\BoxStatusServer($param))->main($body);
//                break;
//            case '5a':
//                //工作开始命令响应
//                $responseBody = (new \App\Servers\BoxResponseServer($param))->main($body);
//                break;
//            case 'a7':
//                //心跳
//                $responseBody = (new \App\Servers\HeartBeatServer($param))->main($body);
//                break;
//            case '6a':
//                //获取位置信息
//                $responseBody = (new \App\Servers\LocationServer($param))->main($body);
//                break;
//        }
//
//        if ($body !== false) {
////            return self::response($body, $type);
//        }
    }

    public static function handleMessage($client_id, $message)
    {
        $data = self::str2arr(strtolower($message));
        $type = $data[1];

        //校检位数
        $length = $data[2];
        $content = array_slice($data, 3, -3);

        if (count($content) !== (int)$length) {
            self::log("msg length is error!");
            return false;
        }

        //校检是否正确
//        $header = array_slice($data, 0, 3);
        $checkCode = array_slice($data, -3, 1)[0];

        $checkCodeRe = self::sumDexArr(array_slice($data, 0, -3));

        if ($checkCodeRe !== $checkCode) {
            self::log("check is error!");
            return false;
        }

        $box_no = self::decodeBoxNo(array_slice($content, 0, 15));
        self::log($box_no);
//        if (!isset($_SESSION['box_no'])) {
        $_SESSION['box_no'] = $box_no;
        self::onLine($_SESSION['box_no']);
//        } else {
//            self::onLine($_SESSION['box_no']);
//        }

//        if (!Gateway::getUidByClientId($client_id)) {
        Gateway::bindUid($client_id, $box_no);
//        }


//        self::response('a5', 'a6');
        $param = [
            'db' => self::$db,
            'redis' => self::$redis
        ];
        $body = $content;
        switch ($type) {
            case 'a5':
                //数据上传
                $responseBody = (new \App\Servers\BoxStatusServer($param))->main($body);
                break;
            case '5a':
                //工作开始命令响应
                $responseBody = (new \App\Servers\BoxResponseServer($param))->main($body);
                break;
            case 'a7':
                //心跳
                $responseBody = (new \App\Servers\HeartBeatServer($param))->main($body);
                break;
            case '6a':
                //获取位置信息
                $responseBody = (new \App\Servers\LocationServer($param))->main($body);
                break;
        }

        if ($body !== false) {
//            return self::response($body, $type);
        }
    }

    /**
     * 当用户断开连接时触发
     * @param int $client_id 连接id
     */
    public static function onClose($client_id)
    {
        $box_no = $_SESSION['box_no'];
        // 离线检查
        if (!Gateway::isUidOnline($box_no)) {
            self::outLine($box_no);
            $_SESSION['box_no'] = null;
        }
    }

    /**
     * 回复
     * @param $body
     * @param $cmd
     * @return bool
     * Author: Mead
     */
    public static function response($body, $cmd)
    {
        $msg = "aa{$cmd}01{$body}";
        $msg .= self::sumDexArr(self::str2arr($msg));
        $msg .= '0d0a';
        return Gateway::sendToCurrentClient(hex2bin($msg));
    }
}
